#	OzCursors.tcl
#
#	There are two cursor modes 
#	1 = wait for selection, 
#	0 = leave in cursor mode  NOT IMPLEMENTED!!!!
#
#	The cursor selection is started by pressing the left button,
#       drawing takes place during motion
#       Final selection is when the button is released
#
#	on final selection and the wait variable has value 99
#	the callback is activated
#	Any other value means the selection has been aborted
#	and the cursor is removed but the callback is not activated.
#
#	cursor type codes are
#       0 = none, disable
#       1 = cross hair (point)
#       2 = box cursor (select both corners)
#       3 = circle type, select centre & outside point
#       4 = straight line (select both ends)
#       5 = fixed size box
#       6 = digitise line

# globals
set ozCursX1 0
set ozCursY1 0
set ozCursX2 0
set ozCursY2 0
set ozDigit 0
set ozCount 0
set ozBoxWidth 0
set ozBoxHeight 0
set ozFinished 0
set ozResult 0

#######=================== X hair ====================================


proc OzStartLoc {c x y} {
	global ozCursX1 ozCursY1  ozFinished
	#puts OzStartLoc
	set ozCursX1 $x
	set ozCursY1 $y

	$c configure -cursor crosshair

}

proc OzDisplayLoc {c x y} {
    	global ozCursX1 ozCursY1
    	set ozCursX1 [$c canvasx $x]
    	set ozCursY1 [$c canvasy $y]
  	#puts "displayLoc $ozCursX1 $ozCursY1"

}

proc OzLocCursor {c xStart yStart} {
	global  ozResult
	#puts OzLocCursor
	bind $c  <Button-1> "OzStartLoc $c %x %y"
	bind $c  <B1-Motion> "OzDisplayLoc $c %x %y"
	bind $c  <ButtonRelease-1> "OzFinishLoc $c"
	set ozFinished 0
	#puts "waiting"
	tkwait variable ozFinished
	#puts "cursor ozFinished"

	OzResults $ozResult
}

proc OzFinishLoc {c} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished ozResult


	set ozCursX1 [expr round($ozCursX1)]
	set ozCursY1 [expr round($ozCursY1)]
	set ozCursX2 0
	set ozCursY2 0
	set ozResult [concat $ozCursX1 $ozCursY1 $ozCursX2 $ozCursY2]

	bind $c  <Button-1> ""
	bind $c  <B1-Motion> ""
	bind $c  <ButtonRelease-1> ""

	$c configure -cursor arrow

	#puts "OzFinishLoc $ozResult"

	set ozFinished 99

}


####======================= box cursors ======================

proc OzStartBox {c x y} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished
	#puts OzStartBox
	set ozCursX1 $x
	set ozCursY1 $y
	set ozCursX2 $x
	set ozCursY2 $y
}

proc OzDisplayBox {c x y} {
    	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 
	#puts OzDisplayBox
    	set x [$c canvasx $x]
    	set y [$c canvasy $y]
  	#puts "displayBox $x $y"
   	if {($ozCursX1 != $x) && ($ozCursY1 != $y)} {
		$c delete ozCurs
		$c addtag ozCurs withtag [$c create rect $ozCursX1 $ozCursY1 $x $y \
		-outline black]
		set ozCursX2 $x
		set ozCursY2 $y
    	}
}



proc OzBoxCursor {c xStart yStart} {
	global  ozResult
	#puts OzBoxCursor
	bind $c  <Button-1> "OzStartBox $c %x %y"
	bind $c  <B1-Motion> "OzDisplayBox $c %x %y"
	bind $c  <ButtonRelease-1> "OzFinishBox $c"
	set ozFinished 0
	#puts "waiting"
	tkwait variable ozFinished
	#puts "cursor ozFinished"

	OzResults $ozResult
}

proc OzFinishBox {c} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished ozResult

	if {$ozCursX1 > $ozCursX2} {
		set temp $ozCursX1
		set ozCursX1 $ozCursX2
		set ozCursX2 $temp
	}
	if {$ozCursY1 < $ozCursY2} {
		set temp $ozCursY1
		set ozCursY1 $ozCursY2
		set ozCursY2 $temp
	}
	set ozCursX1 [expr round($ozCursX1)]
	set ozCursY1 [expr round($ozCursY1)]
	set ozCursX2 [expr round($ozCursX2)]
	set ozCursY2 [expr round($ozCursY2)]
	set ozResult [concat $ozCursX1 $ozCursY1 $ozCursX2 $ozCursY2]
	$c delete ozCurs

	
	bind $c  <Button-1> ""
	bind $c  <B1-Motion> ""
	bind $c  <ButtonRelease-1> ""

	#puts "OzFinishBox $ozResult"

	set ozFinished 99

}


proc OzStartFixedBox {c x y } {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozBoxWidth ozBoxHeight
	#puts OzStartFixedBox
	#puts $ozBoxWidth
	set ozCursX1 $x
	set ozCursY1 $y
	set ozCursX2 [expr $x+$ozBoxWidth]
	set ozCursY2 [expr $y-$ozBoxHeight]

}


proc OzDisplayFixedBox {c x y} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozBoxWidth ozBoxHeight
    	set x [$c canvasx $x]
    	set y [$c canvasy $y]
 	#puts "OzDisplayFixedBox $x $y"
   	if {($ozCursX1 != $x) && ($ozCursY1 != $y)} {
		set ozCursX1 $x
		set ozCursY1 $y
		set ozCursX2 [expr $x+$ozBoxWidth]
		set ozCursY2 [expr $y-$ozBoxHeight]
		$c delete ozCurs
		$c addtag ozCurs withtag [$c create rect \
			$ozCursX1 $ozCursY1 $ozCursX2 $ozCursY2 \
			-outline black]

    	}
}

proc OzFixedBoxCursor {c xStart yStart xEnd yEnd} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozBoxWidth \
		ozBoxHeight ozFinished ozResult
	#puts OzFixedBoxCursor
	set ozBoxWidth [expr $xEnd-$xStart]
	set ozBoxWidth abs($ozBoxWidth)
	set ozBoxHeight [expr $yEnd-$yStart]
	set ozBoxHeight abs($ozBoxHeight)
	bind $c  <Button-1> "OzStartFixedBox $c %x %y"
	bind $c  <B1-Motion> "OzDisplayFixedBox $c %x %y"
	bind $c  <ButtonRelease-1> "OzFinishBox $c"
	set ozFinished 0
	tkwait variable ozFinished

	OzResults $ozResult

}


####======================= circle cursor ======================

proc OzStartCircle {c x y} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished
	#puts OzStartCircle
	set ozCursX1 $x
	set ozCursY1 $y
	set ozCursX2 0
	set ozCursY2 0
}

proc OzDisplayCircle {c x y} {
    	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 
	#puts OzDisplayCircle
    	set x [$c canvasx $x]
    	set y [$c canvasy $y]
	set radsq [expr ($x-$ozCursX1)*($x-$ozCursX1)+($y-$ozCursY1)*($y-$ozCursY1)]
	set rad [expr sqrt($radsq)]
  	#puts "displayCircle $x $y"
   	if {($rad > 0)} {
		$c delete ozCurs
		set x1 [expr $ozCursX1-$rad]
		set y1 [expr $ozCursY1-$rad]
		set x2 [expr $ozCursX1+$rad]
		set y2 [expr $ozCursY1+$rad]
		$c addtag ozCurs withtag [$c create oval $x1 $y1 $x2 $y2 \
		-outline black]
 		set ozCursX2 $x
		set ozCursY2 $y   	}
}



proc OzCircleCursor {c xStart yStart} {
	global  ozResult
	#puts OzCircleCursor
	bind $c  <Button-1> "OzStartCircle $c %x %y"
	bind $c  <B1-Motion> "OzDisplayCircle $c %x %y"
	bind $c  <ButtonRelease-1> "OzFinishCircle $c"
	set ozFinished 0
	#puts "waiting"
	tkwait variable ozFinished
	#puts "cursor ozFinished"

	OzResults $ozResult
}

proc OzFinishCircle {c} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished ozResult

	set ozCursX1 [expr round($ozCursX1)]
	set ozCursY1 [expr round($ozCursY1)]
	set ozCursX2 [expr round($ozCursX2)]
	set ozCursY2 [expr round($ozCursY2)]
	set ozResult [concat $ozCursX1 $ozCursY1 $ozCursX2 $ozCursY2]
	$c delete ozCurs

	
	bind $c  <Button-1> ""
	bind $c  <B1-Motion> ""
	bind $c  <ButtonRelease-1> ""

	#puts "OzFinishCircle $ozResult"

	set ozFinished 99

}




####======================= line cursor ======================

proc OzStartLine {c x y} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished
	#puts OzStartLine
	set ozCursX1 $x
	set ozCursY1 $y
	set ozCursX2 $x
	set ozCursY2 $y
}

proc OzDisplayLine {c x y} {
    	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 
	#puts OzDisplayLine
    	set x [$c canvasx $x]
    	set y [$c canvasy $y]
  	#puts "displayLine $x $y"
   	if {($ozCursX1 != $x) && ($ozCursY1 != $y)} {
		$c delete ozCurs
		$c addtag ozCurs withtag [$c create line $ozCursX1 $ozCursY1 $x $y \
		-fill black]
		set ozCursX2 $x
		set ozCursY2 $y
    	}
}



proc OzLineCursor {c xStart yStart} {
	global  ozResult
	#puts OzLineCursor
	bind $c  <Button-1> "OzStartLine $c %x %y"
	bind $c  <B1-Motion> "OzDisplayLine $c %x %y"
	bind $c  <ButtonRelease-1> "OzFinishLine $c"
	set ozFinished 0
	#puts "waiting"
	tkwait variable ozFinished
	#puts "cursor ozFinished"

	OzResults $ozResult
}

proc OzFinishLine {c} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished ozResult

	if {$ozCursX1 > $ozCursX2} {
		set temp $ozCursX1
		set ozCursX1 $ozCursX2
		set ozCursX2 $temp
	}
	if {$ozCursY1 < $ozCursY2} {
		set temp $ozCursY1
		set ozCursY1 $ozCursY2
		set ozCursY2 $temp
	}
	set ozCursX1 [expr round($ozCursX1)]
	set ozCursY1 [expr round($ozCursY1)]
	set ozCursX2 [expr round($ozCursX2)]
	set ozCursY2 [expr round($ozCursY2)]
	set ozResult [concat $ozCursX1 $ozCursY1 $ozCursX2 $ozCursY2]
	$c delete ozCurs

	
	bind $c  <Button-1> ""
	bind $c  <B1-Motion> ""
	bind $c  <ButtonRelease-1> ""

	#puts "OzFinishLine $ozResult"

	set ozFinished 99

}



### ================ digitise line =======================================



proc OzStartDig {c x y} {
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished
	global ozDigit ozCount
	#puts OzStartDig
	set x [expr round($x)]
	set y [expr round($y)]
	set ozCursX1 $x
	set ozCursY1 $y
	set ozCursX2 $x
	set ozCursY2 $y
	set ozDigit "$x $y $x $y"
	set ozCount 2
}

proc OzDisplayDig {c x y} {
    	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 
	global ozDigit ozCount
	#puts OzDisplayDig
    	set x [$c canvasx $x]
    	set y [$c canvasy $y]
 	set x [expr round($x)]
	set y [expr round($y)]
 	#puts "displayDig $x $y"
   	if {($ozCursX1 != $x) && ($ozCursY1 != $y)} {
		append ozDigit " $x $y"
	#puts $ozDigit
		incr ozCount
		$c delete ozCurs
		$c addtag ozCurs withtag [eval "$c create line $ozDigit -fill black"]
		set ozCursX2 $x
		set ozCursY2 $y
    	}
}



proc OzDigCursor {c xStart yStart} {
	global  ozResult
	global ozDigit ozCount
	#puts OzDigCursor
	bind $c  <Button-1> "OzStartDig $c %x %y"
	bind $c  <B1-Motion> "OzDisplayDig $c %x %y"
	bind $c  <ButtonRelease-1> "OzFinishDig $c"
	set ozFinished 0
	#puts "waiting"
	tkwait variable ozFinished
	#puts "cursor ozFinished"

	OzResults $ozResult
}

proc OzFinishDig {c} {
	global ozDigit ozCount
	global ozCursX1 ozCursY1 ozCursX2 ozCursY2 ozFinished ozResult


	set ozResult "$ozCount $ozDigit"
	$c delete ozCurs

	
	bind $c  <Button-1> ""
	bind $c  <B1-Motion> ""
	bind $c  <ButtonRelease-1> ""

	#puts "OzFinishDig $ozResult"

	set ozFinished 99

}




